@@ -466,18 +466,28 @@ public final class Main
 
         String classOrPackage = (String) it.next();
 
-        if (classOrPackage.endsWith(".java")) {
-          File sourceFile = new File(classOrPackage);
-          if (!sourceFile.exists()) {
-          }
-          else if (sourceFile.isDirectory()) {
+        boolean foundSourceFile = false;
 
+        if (classOrPackage.endsWith(".java")) {
+          for (Iterator pit = option_sourcepath.iterator(); pit.hasNext() && !foundSourceFile; ) {
+            File sourceDir = (File)pit.next();
+            File sourceFile = new File(sourceDir, classOrPackage);
+            if (sourceFile.exists() && !sourceFile.isDirectory()) {
+              rootDoc.addSpecifiedSourceFile(sourceFile);
+              foundSourceFile = true;
+              break;
+            }
           }
-          else {
-            rootDoc.addSpecifiedSourceFile(sourceFile);
+          if (!foundSourceFile) {
+            File sourceFile = new File(classOrPackage);
+            if (sourceFile.exists() && !sourceFile.isDirectory()) {
+              rootDoc.addSpecifiedSourceFile(sourceFile);
+              foundSourceFile = true;
+            } 
           }
         }
-        else {
+
+        if (!foundSourceFile) {
         //--- Check for illegal name
 
         if (classOrPackage.startsWith(".")
