@@ -1,5 +1,5 @@
 /* gnu.classpath.tools.gjdoc.Parser
-   Copyright (C) 2001 Free Software Foundation, Inc.
+   Copyright (C) 2001, 2005 Free Software Foundation, Inc.
 
    This file is part of GNU Classpath.
 
@@ -208,7 +208,6 @@ import gnu.classpath.tools.MalformedInputEvent;
       int process(Parser parser, char[] source, int startIndex, int endIndex) {
 	 String packageName=new String(source,startIndex+8,endIndex-startIndex-8-1).trim();
 	 parser.packageOpened(packageName);
-         parser.importedStatementList.add(packageName + ".*");
 	 return endIndex;
       }
    }
@@ -733,7 +732,6 @@ public class Parser {
       importedStringList.clear();
       importedPackagesList.clear();
       importedStatementList.clear();
-      importedStatementList.add("java.lang.*");
       
       currentLine = 1;
 
@@ -841,6 +839,10 @@ public class Parser {
          }
       }
 
+      if (currentPackageName != null)
+	 importedStatementList.add(currentPackageName + ".*");
+      importedStatementList.add("java.lang.*");
+
       ClassDocImpl classDoc
 	 = ClassDocImpl.createInstance((ctx!=null)?(ctx.classDoc):null, currentPackage, 
 				       null,
